#!/usr/bin/env python
import os
import os.path
import string

# Generates a *.devhelp index file (old format) for man pages.
# Just the index file, this script doesn't generate HTML pages for each man
# page.

# Add $GNOME_PATH/man too
#MAN_PATH = ["/usr/man", "/usr/share/man", "/usr/local/man"]
MAN_PATH = ["/usr/share/man"]
MAN_SUBS = ["man1", "man2", "man3", "man4", "man5",
            "man6", "man7", "man8", "man9"]

MAN_NAME = { "man1" : "Applications",
             "man2" : "System Calls",
	     "man3" : "Library Functions",
	     "man4" : "Devices",
	     "man5" : "Configuration Files",
	     "man6" : "Games",
	     "man7" : "Conventions",
	     "man8" : "System Administration",
	     "man9" : "Kernel Routines"}

# 1. Get a list of all mandirs
chapters = []
for mandir in MAN_PATH:
    dir_list = os.listdir (mandir)
    for dir in dir_list:
	if os.path.isdir (mandir + "/" + dir) and dir in MAN_SUBS:
	    chapters.append (mandir + "/" + dir)

# 2. Get a list of all manpages
man_pages = {}
funcs = []
for mandir in chapters:
    list = os.listdir (mandir)
    for file in list:
	# Extract the name of the man page
	name, section = string.split (file, ".", 1)
	if section[-3:] == ".gz":
	    section = section[:-3]

	man = os.path.basename (mandir)
	filename = mandir + "/" + file

	# Check out section 2* and 3*, but not 3pm
	if section in ['2', '3'] and section[-2:] != "pm":
	    # Remove perl funtions
	    if string.find (name, "::") == -1:
		funcs.append ((name, filename))

	#  Does the the dict have the key?
	if not man_pages.has_key (man):
	    man_pages[man] = []

	if name == "." or not name:
	    continue

	man_pages[man].append ((name, filename))

# 3. Print out xml
print '<?xml version="1.0"?>'
print '<book title="Man pages">'
print

# 3.1 Print chapters
# Chapters
chapters.sort()
written = []
print '<chapters>'
for chap in chapters:
    name = os.path.basename (chap)
    if not man_pages.has_key (name):
	continue

    if name in written:
	continue
    written.append (name)

    print '  <chapter name="%s">' % MAN_NAME[name]
    list = man_pages[name]
    list.sort()

    for name, page in list:
	print '    <sub name="%s" link="%s"/>' % (name, page)
    print '  </chapter>'
    print

print '</chapters>'
print

# 3.2 print functions
# Functions
funcs.sort()
print '<functions>'
for name, file in funcs:
    print '  <function name="%s" link="%s"/>' % (name, file)
print '</functions>'
print
print '</book>'





#!/usr/bin/env python
import os.path
import sgmllib
import string
import sys

def does_dict_have_keys (dict, keys):
    for key in keys:
	if not dict.has_key (key):
	    return 0
    if len(dict) != len(keys):
	return 0
    return 1

def walk (dict, level=0, parent=None):
    if dict.has_key ('order'):
	list = dict['order']
    else:
	list = dict.keys()

    for key in list:
	if key in ['name', 'order', 'link']:
	    continue
	if dict[key].has_key ('link') and  \
  	   does_dict_have_keys (dict[key], ['link']):
	    link = dict[key]['link']
	else:
	    link = ""

	if level:
	    print '*' * level, key, '-', link
	else:
	    print key, '-', link

	walk (dict[key], level + 1, dict)

class BookParser (sgmllib.SGMLParser):
    def __init__ (self):
	sgmllib.SGMLParser.__init__ (self)
	self.a = self.parents = []
	self.dict = {}
	self.last = self.link = ""
	self.is_a = self.level = 0
	self.first = 1

    def unknown_starttag (self, tag, attrs):
	if tag == 'a':
	    self.is_a = 1
	    for attr in attrs:
		if attr[0] == "href":
		    self.link = attr[1]
		    break

	if tag in ['dd', 'ul']:
	    self.parents.append (self.last)
	    self.level = self.level + 1

    def unknown_endtag (self, tag):
	if tag == 'a':
	    self.is_a = 0

	if tag in ['dd', 'ul']:
	    self.level = self.level - 1
	    self.parents.pop()

    def handle_data (self, data):
	data = string.strip (data)
	if not data or data in [ ">", "<" ]:
	    return

	if self.first:
	    self.dict['name'] = data
	    self.first = 0
	    return

	if data == self.dict['name'] or data in [ "Next Page", "Previous Page", "Home", "Next"]:
	    return

	if len (self.parents) == 0:
	    dict = self.dict
	elif len (self.parents) == 1:
	    dict = self.dict[self.parents[0]]
	elif len (self.parents) == 2:
	    dict = self.dict[self.parents[0]][self.parents[1]]
	elif len (self.parents) == 3:
	    dict = self.dict[self.parents[0]][self.parents[1]][self.parents[2]]
	else:
	    dict = None

	if self.is_a:
	    if dict == None:
		return

	    if not dict.has_key (data):
		dict[data] = {}
	    if not dict.has_key ('order'):
		dict['order'] = []
	    dict['order'].append (data)
	    dict[data]['link'] = self.link

	    self.last = data

def parse_book (url):
    if os.path.exists (url + "/index.html"):
	filename = url + "/index.html"
    elif os.path.exists (url + "/book1.html"):
	filename = url + "/book1.html"
    elif os.path.exists (url):
	filename = url
    else:
	print "Error; Can't find an index :("
	raise SystemExit

    fd = open (filename)
    p = BookParser()
    p.feed (fd.read())
    p.close()
    return p.dict

filename = sys.argv[1]

dict =  parse_book (sys.argv[1])

print '<?xml version="1.0"?>'
print '<book title="%s"\nname=""\nbase=""\nlink="%s">' % (dict['name'], os.path.basename (sys.argv[1]))

print '<chapters>'
for chap in dict['order']:
    print '  <sub name="%s" link="%s">' % (chap, dict[chap]['link'])
    if dict[chap].has_key ('order'):
        for sub in dict[chap]['order']:
            if not does_dict_have_keys (dict[chap][sub], ['link']):
                print '    <sub name="%s" link="%s">' % (sub, dict[chap][sub]['link'])

                for sub2 in dict[chap][sub]['order']:
                    print '      <sub name="%s" link="%s"/>' % (sub2, dict[chap][sub][sub2]['link'])
                print '    </sub>'
            else:
                print '    <sub name="%s" link="%s"/>' % (sub, dict[chap][sub]['link'])

    print '  </sub>'
    print

print '</chapters>'
print
print '</book>'




#!/usr/bin/env python3

import os
import subprocess
import sys

data_dir = sys.argv[1]

if not os.environ.get('DESTDIR'):
    icon_dir = os.path.join(data_dir, 'icons', 'hicolor')
    print('Update icon cache...')
    subprocess.call(['gtk-update-icon-cache', '-f', '-t', icon_dir])

    schema_dir = os.path.join(data_dir, 'glib-2.0', 'schemas')
    print('Compiling gsettings schemas...')
    subprocess.call(['glib-compile-schemas', schema_dir])
